@@ -6,6 +6,8 @@
 Make nice spectrum objects to pass around SED class
 """
 import copy
+from functools import wraps
+import os
 from pkg_resources import resource_filename
 
 import astropy.constants as ac
@@ -22,6 +24,23 @@ from pandas import DataFrame
 from . import utilities as u
 
 
+def copy_raw(func):
+    """A wrapper to copy the raw data to the new Spectrum object"""
+    @wraps(func)
+    def _copy_raw(*args, **kwargs):
+        """Run the function then update the raw data attribute"""
+        # Grab the original data
+        raw = args[0].raw or args[0].spectrum
+
+        # Run the function and update the raw attribute
+        new_spec = func(*args, **kwargs)
+        new_spec.raw = raw
+
+        return new_spec
+
+    return _copy_raw
+
+
 class Spectrum:
     """
     A class to store, calibrate, fit, and plot a single spectrum
@@ -83,9 +102,9 @@ class Spectrum:
         spectrum += [unc] if unc is not None else []
         spectrum = u.scrub(spectrum, fill_value=np.nan)
 
-        # Store smoothing info and raw data
-        self.smoothing = None
-        self.raw = spectrum
+        # Store history info and raw data
+        self.history = {}
+        self.raw = None
 
         # Strip and store units
         self._wave_units = wave.unit
@@ -112,6 +131,7 @@ class Spectrum:
         for key, val in kwargs.items():
             setattr(self, key, val)
 
+    @copy_raw
     def __add__(self, spec):
         """Add the spectra of this and another Spectrum object
 
@@ -293,6 +313,30 @@ class Spectrum:
 
         return data
 
+    def export(self, filepath, header=None):
+        """
+        Export the spectrum to file
+
+        Parameters
+        ----------
+        filepath: str
+            The path for the exported file
+        """
+        # Get target directory
+        dirname = os.path.dirname(filepath) or '.'
+        name = self.name.replace(' ', '_')
+
+        # Check the parent directory
+        if not os.path.exists(dirname):
+            raise IOError('{}: No such directory'.format(dirname))
+
+        # Write the file with a header
+        head = '{}\nWavelength [{}], Flux Density [{}]'.format(name, self.wave_units, self.flux_units)
+        if isinstance(header, str):
+            head += '\n{}'.format(header)
+        t_data = np.asarray(self.spectrum).T
+        np.savetxt(filepath, t_data, header=head)
+
     def fit(self, spec, weights=None, wave_units=None, scale=True, plot=False):
         """Determine the goodness of fit between this and another spectrum
 
@@ -363,6 +407,7 @@ class Spectrum:
 
         return gstat, ynorm, xnorm
 
+    @copy_raw
     def flux_calibrate(self, distance, target_distance=10 * q.pc, flux_units=None):
         """Flux calibrate the spectrum from the given distance to the target distance
 
@@ -455,6 +500,7 @@ class Spectrum:
 
         return val, unc
 
+    @copy_raw
     def interpolate(self, wave):
         """Interpolate the spectrum to another wavelength array
 
@@ -486,6 +532,7 @@ class Spectrum:
 
         return Spectrum(wave, f1, e1, name=self.name)
 
+    @copy_raw
     def norm_to_mags(self, photometry, force=False, exclude=[], include=[]):
         """
         Normalize the spectrum to the given bandpasses
@@ -569,6 +616,7 @@ class Spectrum:
 
         return Spectrum(*spectrum, name=self.name)
 
+    @copy_raw
     def norm_to_spec(self, spec, add=False, plot=False, **kwargs):
         """Normalize the spectrum to another spectrum
 
@@ -713,6 +761,7 @@ class Spectrum:
 
         return Spectrum(*spectrum, name=self.name)
 
+    @copy_raw
     def resamp(self, wave=None, resolution=None):
         """Resample the spectrum onto a new wavelength array or to a new
         resolution
@@ -754,6 +803,13 @@ class Spectrum:
 
         return Spectrum(*spectrum, name=self.name)
 
+    def restore(self):
+        """
+        Restore the spectrum to the original raw data
+        """
+        return Spectrum(*(self.raw or self.spectrum), name=self.name)
+
+    @copy_raw
     def smooth(self, beta, window=11):
         """
         Smooths the spectrum using a Kaiser-Bessel smoothing window of
@@ -780,7 +836,7 @@ class Spectrum:
         spectrum = self.spectrum
         spectrum[1] = smoothed * self.flux_units
 
-        return Spectrum(*spectrum, name=self.name, smoothing=(beta, window))
+        return Spectrum(*spectrum, name=self.name)
 
     @property
     def size(self):
@@ -870,13 +926,16 @@ class Spectrum:
         tuple
             The flux and uncertainty
         """
-        flx = self.synthetic_flux(bandpass, force=force)
+        flx, flx_unc = self.synthetic_flux(bandpass, force=force)
 
         # Calculate the magnitude
-        mag = u.flux2mag(flx, bandpass)
+        mag, mag_unc = None, None
+        if flx is not None:
+            mag, mag_unc = u.flux2mag((flx, flx_unc), bandpass)
 
-        return mag
+        return mag, mag_unc
 
+    @copy_raw
     def trim(self, ranges):
         """Trim the spectrum in the given wavelength ranges
 
@@ -885,26 +944,24 @@ class Spectrum:
         ranges: sequence
             The (min_wave, max_wave) ranges to trim from the spectrum
         """
+        # Iterate over trim ranges
         if isinstance(ranges, (list, tuple)):
             for mn, mx in ranges:
                 try:
                     idx, = np.where((self.spectrum[0] < mn) | (self.spectrum[0] > mx))
 
                     if len(idx) > 0:
+
+                        # Trim the data
                         spectrum = [i[idx] for i in self.spectrum]
 
-                        # Update the object
-                        spec = Spectrum(*spectrum, name=self.name)
-
-                        return spec
+                        return Spectrum(*spectrum, name=self.name)
 
                 except TypeError:
-                    print("""Please provide a list of (lower,upper) bounds\
-                             with units to trim, e.g. [(0*q.um,0.8*q.um)]""")
+                    print("""Please provide a list of (lower,upper) bounds with units to trim, e.g. [(0*q.um,0.8*q.um)]""")
 
         else:
-            raise TypeError("""Please provide a list of (lower,upper) bounds\
-                             with units to trim, e.g. [(0*q.um,0.8*q.um)]""")
+            raise TypeError("""Please provide a list of (lower,upper) bounds with units to trim, e.g. [(0*q.um,0.8*q.um)]""")
 
     @property
     def wave_max(self):
@@ -1130,8 +1187,7 @@ class FileSpectrum(Spectrum):
 
 class Vega(Spectrum):
     """A Spectrum object of Vega"""
-    def __init__(self, wave_units=q.AA, flux_units=q.erg / q.s / q.cm**2 / q.AA,
-                 **kwargs):
+    def __init__(self, wave_units=q.AA, flux_units=q.erg / q.s / q.cm**2 / q.AA, **kwargs):
         """Initialize the Spectrum object
 
         Parameters
