@@ -97,6 +97,14 @@ class TestSpectrum(unittest.TestCase):
         spec4 = self.spec + None
         self.assertEqual(spec4.size, self.spec.size)
 
+    def test_export(self):
+        """Test export method"""
+        # Good export
+        self.flat1.export('test.txt', header='Foo')
+
+        # Bad dirname
+        self.assertRaises(IOError, self.flat1.export, '/foo/bar/baz.txt')
+
     def test_integrate(self):
         """Test that a spectum is integrated properly"""
         # No nans
@@ -146,6 +154,17 @@ class TestSpectrum(unittest.TestCase):
         self.assertEqual(self.spec.wave_units, new_spec.wave_units)
         self.assertNotEqual(self.spec.size, new_spec.size)
 
+    def test_restore(self):
+        """Test restore method"""
+        # Smooth the spectrum
+        edited_spec = self.spec.smooth(5)
+
+        # Restore it
+        restored_spec = edited_spec.restore()
+
+        # Make sure it matched the original
+        self.assertTrue(np.all(self.spec.wave == restored_spec.wave))
+
     def test_norm_to_spec(self):
         """Test that a spectrum is properly normalized to another spectrum"""
         # Get two flat spectra
